#pragma once

// AronMacros (copied from Config.h)
/************************************
 * TYPES ****************************
 ***********************************/
#define HANDLE_PRIMITIVE_TYPES \
    RUN_ARON_MACRO(Int, int, INT) \
    RUN_ARON_MACRO(Long, long, LONG) \
    RUN_ARON_MACRO(Float, float, FLOAT) \
    RUN_ARON_MACRO(Double, double, DOUBLE) \
    RUN_ARON_MACRO(String, string, STRING) \
    RUN_ARON_MACRO(Bool, bool, BOOL) \
    RUN_ARON_MACRO(Time, time, TIME)

/************************************
 * DATA *****************************
 ***********************************/
#define HANDLE_PRIMITIVE_DATA \
    RUN_ARON_MACRO(Int, int, INT) \
    RUN_ARON_MACRO(Long, long, LONG) \
    RUN_ARON_MACRO(Float, float, FLOAT) \
    RUN_ARON_MACRO(Double, double, DOUBLE) \
    RUN_ARON_MACRO(String, string, STRING) \
    RUN_ARON_MACRO(Bool, bool, BOOL)

module armarx
{
    module aron
    {

        /*************************
         * General Definitions ***
         ************************/
        sequence<byte> AronByteSequence;
        sequence<int> AronIntSequence;
        sequence<string> AronStringSequence;
        dictionary<string, int> AronIntDict;


        /*************************
         * Aron Types ************
         ************************/
        module type
        {
            class AronType { };
            sequence<AronType> AronTypeList;
            dictionary<string, AronType> AronTypeDict;

            // Container types
            class AronList extends AronType { AronType acceptedType; };
            class AronTuple extends AronType { AronTypeList elementTypes; };
            class AronPair extends AronType { AronType acceptedType1; AronType acceptedType2; };
            class AronObject extends AronType { AronObject parent; string objectName; AronTypeDict elementTypes;};
            class AronDict extends AronType { AronType acceptedType; };

            // Complex Types (serialize to ndarray)
            class AronEigenMatrix extends AronType { AronIntSequence dimensions; string typeName; };
            class AronEigenQuaternion extends AronType { string typeName; }
            class AronIVTCByteImage extends AronType { int width; int height; string typeName; }
            class AronOpenCVMat extends AronType { AronIntSequence dimensions; string typeName; }
            class AronPCLPointCloud extends AronType { int width; int height; string typeName; }
            class AronPosition extends AronType { }
            class AronOrientation extends AronType { }
            class AronPose extends AronType { }

            // Enum types
            class AronIntEnum extends AronType { string enumName; AronIntDict acceptedValues; }

            // Primitive Types
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType extends AronType { };

                    HANDLE_PRIMITIVE_TYPES
            #undef RUN_ARON_MACRO
        };


        /*************************
         * Aron Data *************
         ************************/
        module data
        {
            class AronData { };
            sequence<AronData> AronDataList;
            dictionary<string, AronData> AronDataDict;

            // Container Data
            class AronList extends AronData { AronDataList elements; };
            class AronDict extends AronData { AronDataDict elements; };

            // Complex Data. The NDArray contains more or less the same information as an AronType, but there is no other way to do it
            // Especially, note the difference between the type's typeName (e.g. "GRAY_SCALE_IMAGE") and the data's type ("0")
            // Further, note the difference between the type's dimensions (e.g. 128x128) and the data's dimensions (128x128x3 for RGB)
            class AronNDArray extends AronData { AronIntSequence dimensions; string type; AronByteSequence data; };

            // Primitive Data
            #define RUN_ARON_MACRO(upperType, lowerType, capsType) \
                class Aron##upperType extends AronData { lowerType value; };

                    HANDLE_PRIMITIVE_DATA
            #undef RUN_ARON_MACRO

            // useful for memory ice_conversions
            sequence<AronDict> AronDictSeq;
        };

    };
};

#undef HANDLE_PRIMITIVE_DATA
#undef HANDLE_PRIMITIVE_TYPES
