#include "RobotUnitComponentPlugin.h"

namespace armarx::plugins
{
    RobotUnitDataStreamingReceiverPtr
    RobotUnitComponentPlugin::startDataSatreming(
        const RobotUnitDataStreaming::Config& cfg)
    {
        //ok to create smart ptr from parent, since ice handels this
        return make_shared<RobotUnitDataStreamingReceiver>(
                   &parent(), getRobotUnit(), cfg);
    }

    void RobotUnitComponentPlugin::postOnDisconnectComponent()
    {
        if (!_ctrls.empty())
        {
            ARMARX_CHECK_NOT_NULL(_robotUnit);
            std::vector<std::string> cs(_ctrls.begin(), _ctrls.end());
            _robotUnit->deactivateAndDeleteNJointControllers(cs);
        }
        _robotUnit = nullptr;
    }

    NJointControllerInterfacePrx RobotUnitComponentPlugin::createNJointController(
        const std::string& className,
        const std::string& instanceName,
        const armarx::NJointControllerConfigPtr& config,
        bool doManageController)
    {
        ARMARX_CHECK_NOT_NULL(_robotUnit);
        ARMARX_INFO << ARMARX_STREAM_PRINTER
        {
            out << "creating ";
            if (doManageController)
            {
                out << "and managing ";
            }
            out << " controller '" << instanceName
                << "' of class '" << instanceName << "'";
        };
        const auto prx = _robotUnit->createOrReplaceNJointController(className, instanceName, config);
        ARMARX_CHECK_NOT_NULL(prx);
        if (doManageController)
        {
            manageController(instanceName);
        }
        return prx;
    }

    void RobotUnitComponentPlugin::manageController(const armarx::NJointControllerInterfacePrx& ctrl)
    {
        ARMARX_CHECK_NOT_NULL(ctrl);
        manageController(ctrl->getInstanceName());
    }

    void RobotUnitComponentPlugin::manageController(const std::string& ctrl)
    {
        _ctrls.emplace(ctrl);
    }

    void RobotUnitComponentPlugin::preOnInitComponent()
    {
        if (!_deactivated)
        {
            if (_robotUnitName.empty())
            {
                parent<Component>().getProperty(_robotUnitName, PROPERTY_NAME);
            }
            parent<Component>().usingProxy(_robotUnitName);
        }
    }

    void RobotUnitComponentPlugin::preOnConnectComponent()
    {
        if (!_deactivated)
        {
            parent<Component>().getProxy(_robotUnit, _robotUnitName);
        }
    }

    void RobotUnitComponentPlugin::postCreatePropertyDefinitions(armarx::PropertyDefinitionsPtr& properties)
    {
        if (!_deactivated && !properties->hasDefinition(PROPERTY_NAME))
        {
            properties->defineRequiredProperty<std::string>(
                PROPERTY_NAME,
                "Name of the RobotUnit");
        }
    }

    RobotUnitInterfacePrx RobotUnitComponentPlugin::getRobotUnit() const
    {
        return _robotUnit;
    }

    void RobotUnitComponentPlugin::setRobotUnitName(const std::string& name)
    {
        if (!_deactivated)
        {
            ARMARX_CHECK_NOT_EMPTY(name);
            ARMARX_CHECK_EMPTY(_robotUnitName);
            _robotUnitName = name;
        }
    }
    const std::string& RobotUnitComponentPlugin::getRobotUnitName() const
    {
        return _robotUnitName;
    }

    void RobotUnitComponentPlugin::deactivate()
    {
        _deactivated = true;
    }
}

namespace armarx
{
    RobotUnitComponentPluginUser::RobotUnitComponentPluginUser()
    {
        addPlugin(plugin);
    }

    RobotUnitInterfacePrx RobotUnitComponentPluginUser::getRobotUnit() const
    {
        return plugin->getRobotUnit();
    }

    plugins::RobotUnitComponentPlugin& RobotUnitComponentPluginUser::getRobotUnitComponentPlugin()
    {
        return *plugin;
    }
}


