#pragma once

#include <map>
#include <string>

#include "ProviderSegment.h"
#include "detail/TypedEntityContainer.h"
#include "detail/MaxHistorySize.h"


namespace armarx::armem
{

    /**
     * @brief Data of a core segment containing multiple provider segments.
     */
    class CoreSegment :
        public detail::TypedEntityContainer<ProviderSegment, CoreSegment>
        , public detail::MaxHistorySize
    {
        using Base = detail::TypedEntityContainer<ProviderSegment, CoreSegment>;

    public:

        CoreSegment();
        CoreSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        CoreSegment(const std::string& name, const MemoryID& parentID, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        CoreSegment(const MemoryID& id, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);

        CoreSegment(const CoreSegment& other);
        CoreSegment& operator=(const CoreSegment& other);

        ~CoreSegment() override;


        using Base::name;
        const std::string& name() const override;


        bool hasProviderSegment(const std::string& name) const;
        ProviderSegment& getProviderSegment(const std::string& name);
        const ProviderSegment& getProviderSegment(const std::string& name) const;


        using Base::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;


        using Base::update;
        virtual MemoryID update(const EntityUpdate& update) override;

        /**
         * @brief Add an empty provider segment with the given name and optional provider segment type.
         * @param name The segment name.
         * @param providerSegmentType The provider type. If nullptr, the core segment type is used.
         * @return The added provider segment.
         */
        ProviderSegment& addProviderSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr providerSegmentType = nullptr);
        /// Copy and insert a provider segment.
        ProviderSegment& addProviderSegment(const ProviderSegment& providerSegment);
        /// Move and insert a provider segment.
        ProviderSegment& addProviderSegment(ProviderSegment&& providerSegment);


        /**
         * @brief Sets the maximum history size of entities in this segment.
         * This affects all current entities as well as new ones.
         * @see Entity::setMaxHistorySize()
         */
        void setMaxHistorySize(long maxSize) override;


        bool equalsDeep(const CoreSegment& other) const;


        // MemoryItem interface
    public:
        std::string getLevelName() const override;

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override;
        void _copyElements(CoreSegment& other, bool withData) const override;


    public:

        /// The provider segments.
        std::map<std::string, ProviderSegment> providerSegments;

    };

}
