#include "EntityInstance.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem
{

    EntityInstance::EntityInstance()
    {
    }

    EntityInstance::EntityInstance(int index, const MemoryID& parentID) :
        EntityInstance(parentID.withInstanceIndex(index))
    {
    }

    EntityInstance::EntityInstance(const MemoryID& id) : MemoryItem(id)
    {
    }


    bool EntityInstance::equalsDeep(const EntityInstance& other) const
    {
        return _metadata == other.metadata() && _data->equalsDeep(other.data());
    }

    void EntityInstance::update(const EntityUpdate& update, int index)
    {
        ARMARX_CHECK_FITS_SIZE(index, update.instancesData.size());

        this->index() = index;
        setData(update.instancesData.at(size_t(index)));

        this->_metadata.confidence = update.confidence;

        this->_metadata.timeCreated = update.timeCreated;
        this->_metadata.timeSent = update.timeSent;
        this->_metadata.timeArrived = update.timeArrived;
    }

    EntityInstance EntityInstance::copy(bool withData) const
    {
        EntityInstance other(id());
        other._metadata = _metadata;
        if (withData)
        {
            other.setData(_data);
        }
        return other;
    }

    std::string EntityInstance::getKeyString() const
    {
        return std::to_string(index());
    }

    std::string EntityInstance::getLevelName() const
    {
        return "entity instance";
    }


    bool EntityInstanceMetadata::operator==(const EntityInstanceMetadata& other) const
    {
        return timeCreated == other.timeCreated
               && timeSent == other.timeSent
               && timeArrived == other.timeArrived
               && std::abs(confidence - other.confidence) < 1e-6f;
    }

}
