#pragma once

#include <map>
#include <string>

#include "Entity.h"
#include "detail/TypedEntityContainer.h"
#include "detail/MaxHistorySize.h"


namespace armarx::armem
{

    /**
     * @brief Data of a provider segment containing multiple entities.
     */
    class ProviderSegment :
        public detail::TypedEntityContainer<Entity, ProviderSegment>
        , public detail::MaxHistorySize

    {
        using Base = detail::TypedEntityContainer<Entity, ProviderSegment>;


    public:

        ProviderSegment();
        ProviderSegment(const std::string& name, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        ProviderSegment(const std::string& name, const MemoryID parentID, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        ProviderSegment(const MemoryID id, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);

        ProviderSegment(const ProviderSegment& other);
        ProviderSegment& operator=(const ProviderSegment& other);

        ~ProviderSegment() override;


        using Base::name;
        const std::string& name() const override;


        bool hasEntity(const std::string& name) const;

        using Base::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;

        Entity& getEntity(const std::string& name);
        const Entity& getEntity(const std::string& name) const;



        using Base::update;
        /**
         * @brief Updates an entity's history.
         *
         * Missing entity entries are added before updating.
         */
        virtual MemoryID update(const EntityUpdate& update) override;

        /// Add an empty entity with the given name.
        Entity& addEntity(const std::string& name);
        /// Copy and insert an entity.
        Entity& addEntity(const Entity& entity);
        /// Move and insert an entity.
        Entity& addEntity(Entity&& entity);


        /**
         * @brief Sets the maximum history size of entities in this segment.
         * This affects all current entities as well as new ones.
         * @see Entity::setMaxHistorySize()
         */
        void setMaxHistorySize(long maxSize) override;


        bool equalsDeep(const ProviderSegment& other) const;


        // MemoryItem interface
    public:
        std::string getLevelName() const override;

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override;
        void _copySelf(ProviderSegment& other) const override;
        void _copyElements(ProviderSegment& other, bool withData) const override;


    public:

        /// The entities.
        std::map<std::string, Entity> entities;

    };

}
