/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <filesystem>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/container/Dict.h>
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>
#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>


// BaseClass
#include "../DiskReaderWriter.h"
#include "../FileSystemLookupMemory.h"

#include <RobotAPI/libraries/armem/core/Memory.h>


namespace armarx::armem::io
{
    struct DiskWriterReturnInformation;


    class DiskWriter;
    using DiskWriterPtr = std::shared_ptr<DiskWriter>;

    class DiskWriter : virtual public DiskReaderWriter
    {
    public:

        using DiskReaderWriter::DiskReaderWriter;

        DiskWriterReturnInformation writeOnDisk(const Memory& m);
        DiskWriterReturnInformation writeOnDisk(const MemoryID&, const CoreSegment&);
        DiskWriterReturnInformation writeOnDisk(const MemoryID&, const ProviderSegment&);
        DiskWriterReturnInformation writeOnDisk(const MemoryID&, const Entity&);
        DiskWriterReturnInformation writeOnDisk(const MemoryID&, const EntitySnapshot&);

    protected:
        aron::datanavigator::DictNavigatorPtr wrapData(const EntityInstance& e) const;
        aron::typenavigator::ObjectNavigatorPtr wrapType(const aron::typenavigator::ObjectNavigatorPtr&) const;

        virtual std::string getDataAsString(const aron::datanavigator::DictNavigatorPtr&) const = 0;
        virtual std::string getTypeAsString(const aron::typenavigator::ObjectNavigatorPtr&) const = 0;

        bool directoryPathExists(const std::filesystem::path& p) const;
        bool ensureDirectoryPathExists(const std::filesystem::path& p) const;

        bool filePathExists(const std::filesystem::path& p) const;
    };


    struct DiskWriterReturnInformation
    {
        std::string memoryName = "";
        bool hasError = false;

        FileSystemLookupMemory storedElements;
        std::vector<std::string> coreSegmentsError;
        std::vector<std::string> providerSegmentsError;
        std::vector<std::string> entitiesError;
        std::vector<std::string> historyTimestampsError;
        std::vector<std::string> entityInstancesError;

        DiskWriterReturnInformation(const std::string& m);

        void append(const DiskWriterReturnInformation& o);
    };
}
