#pragma once

#include <mutex>

#include <ArmarXCore/core/Component.h>

#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/armem/client/MemoryListenerInterface.h>
#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>

#include "../core/Memory.h"
#include "../mns/ClientPlugin.h"
#include "MemoryToIceAdapter.h"


namespace armarx::armem::server
{
    class ComponentPluginUser;
}


namespace armarx::armem::server::plugins
{

    class ComponentPlugin : public mns::plugins::ClientPlugin
    {
    public:

        using ClientPlugin::ClientPlugin;
        virtual ~ComponentPlugin() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void postOnConnectComponent() override;
        void preOnDisconnectComponent() override;


        /**
         * @brief Register the parent component in the MNS.
         * Called before onConnect() if MNS is enabled.
         */
        data::RegisterMemoryResult registerMemory(ComponentPluginUser& parent);
        /**
         * @brief Remove the parent component from the MNS.
         * Called before onDisconnect() if MNS is enabled.
         */
        data::RemoveMemoryResult removeMemory(ComponentPluginUser& parent);

        client::MemoryListenerInterfacePrx memoryListener;
    };

}


namespace armarx::armem::server
{

    /**
     * @brief Base class of memory server components.
     */
    class ComponentPluginUser :
        virtual public ManagedIceObject
        , virtual public MemoryInterface
        , virtual public mns::plugins::ClientPluginUserBase
    {
    public:

        ComponentPluginUser();
        virtual ~ComponentPluginUser() override = default;


        // WritingInterface interface
        virtual data::AddSegmentsResult addSegments(const data::AddSegmentsInput& input, const Ice::Current& = Ice::emptyCurrent) override;
        data::AddSegmentsResult addSegments(const data::AddSegmentsInput& input, bool addCoreSegments);

        virtual data::CommitResult commit(const data::Commit& commit, const Ice::Current& = Ice::emptyCurrent) override;


        // ReadingInterface interface
        virtual armem::query::data::Result query(const armem::query::data::Input& input, const Ice::Current& = Ice::emptyCurrent) override;


    public:

        /// The actual memory.
        Memory memory;
        std::mutex memoryMutex;

        /// property defauls
        std::string memoryListenerDefaultName = "MemoryUpdates";

        /// Helps connecting `memory` to ice. Used to handle Ice callbacks.
        MemoryToIceAdapter iceMemory { &memory };


    private:

        plugins::ComponentPlugin* plugin = nullptr;


    };


}
