#pragma once

#include <sstream>
#include <stack>

#include <QTreeWidget>
#include <QLabel>

#include <RobotAPI/libraries/aron/core/navigator/visitors/DataVisitor.h>


namespace armarx::armem::gui
{

    class TreeDataVisitorBase
    {
    public:

        TreeDataVisitorBase();
        TreeDataVisitorBase(QTreeWidgetItem* root);
        virtual ~TreeDataVisitorBase();

        void setColumns(int key, int value, int type);


    protected:

        // Same for Dict and List
        bool _visitEnter(const std::string& key, const std::string& type, size_t numChildren);
        bool _visitExit();


        template <class Navigator>
        bool addValueRow(const std::string& key, Navigator& n, const std::string& typeName)
        {
            if (items.size() > 0)
            {
                items.top()->addChild(new QTreeWidgetItem(this->makeValueRowStrings(key, n, typeName)));
            }
            return true;
        }

        template <class Navigator>
        QStringList makeValueRowStrings(const std::string& key, Navigator& n, const std::string& typeName)
        {
            std::stringstream value;
            streamValueText(n, value);
            QStringList cols;
            cols.insert(columnKey, QString::fromStdString(key));
            cols.insert(columnValue, QString::fromStdString(value.str()));
            cols.insert(columnType, QString::fromStdString(typeName));
            return cols;
        }

        template <class Navigator>
        void streamValueText(Navigator& n, std::stringstream& ss)
        {
            ss << n.getValue();
        }
        void streamValueText(aron::visitor::DataVisitor::BoolDataNavigator& n, std::stringstream& ss);
        void streamValueText(aron::visitor::DataVisitor::StringDataNavigator& n, std::stringstream& ss);
        void streamValueText(aron::visitor::DataVisitor::NDArrayDataNavigator& n, std::stringstream& ss);


    public:

        std::stack<QTreeWidgetItem*> rootItems;
        std::stack<QTreeWidgetItem*> items;

        int columnKey = 0;
        int columnValue = 1;
        int columnType = 2;

    };

}
