#include "TreeTypedDataVisitor.h"

#include <SimoxUtility/algorithm/string.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::gui
{

    std::string TreeTypedDataVisitor::sanitizeTypeName(const std::string& typeName) const
    {
        std::string n = typeName;
        n = simox::alg::replace_all(n, "Aron", "");
        n = simox::alg::replace_all(n, "Type", "");
        return n;
    }

    QStringList TreeTypedDataVisitor::serializePath(const aron::Path& path)
    {
        QStringList qpath;
        qpath.append(QString::fromStdString(path.getRootIdentifier()));
        qpath.append(QString::fromStdString(path.getDelimeter()));
        for (const std::string& item : path.getPath())
        {
            qpath.append(QString::fromStdString(item));
        }
        return qpath;
    }

    aron::Path TreeTypedDataVisitor::deserializePath(const QStringList& qpath)
    {
        ARMARX_CHECK_GREATER_EQUAL(qpath.size(), 2);
        std::vector<std::string> pathItems;
        for (int i = 2; i < qpath.size(); ++i)
        {
            pathItems.push_back(qpath.at(i).toStdString());
        }
        aron::Path path(qpath.at(0).toStdString(), qpath.at(1).toStdString(), pathItems);
        return path;
    }




}
