#include "aron_conversions.h"

#include <algorithm>
#include <iterator>

#include <RobotAPI/interface/units/LaserScannerUnit.h>
#include <RobotAPI/libraries/armem_robot_mapping/aron/LaserScan.aron.generated.h>

#include "types.h"

namespace armarx
{

    /************ fromAron ************/


    LaserScanStep fromAron(const aron::LaserScanStep& aronLaserScanStep)
    {
        LaserScanStep laserScanStep;

        laserScanStep.angle = aronLaserScanStep.angle;
        laserScanStep.distance = aronLaserScanStep.distance;

        return laserScanStep;
    }

    template <typename T> auto fromAron(const std::vector<T>& v)
    {
        std::vector<decltype(fromAron(T()))> r;
        r.reserve(v.size());

        std::transform(v.begin(), v.end(), std::back_inserter(r),
                       [](const T & t)
        {
            return fromAron(t);
        });

        return r;
    }

    auto fromAron(const aron::LaserScan& aronLaserScan)
    {
        return fromAron(aronLaserScan.scan);
    }

    SensorHeader fromAron(const aron::SensorHeader& aronSensorHeader)
    {
        return {.agent = aronSensorHeader.agent,
                .frame = aronSensorHeader.frame,
                .timestamp = aronSensorHeader.timestamp};
    }

    void fromAron(const aron::LaserScanStamped& aronLaserScan,
                  LaserScanStamped& laserScan)
    {
        laserScan.header = fromAron(aronLaserScan.header);
        laserScan.data = fromAron(aronLaserScan.data);
    }

    void fromAron(const aron::LaserScanStamped& aronLaserScan, LaserScan& laserScan,
                  std::int64_t& timestamp, std::string& frame,
                  std::string& agentName)
    {
        const auto header = fromAron(aronLaserScan.header);

        laserScan = fromAron(aronLaserScan.data);

        timestamp = header.timestamp;
        frame = header.frame;
        agentName = header.agent;
    }


    /************ toAron ************/


    template <typename T> auto toAron(const std::vector<T>& v)
    {
        std::vector<decltype(toAron(T()))> r;
        r.reserve(v.size());

        std::transform(v.begin(), v.end(), std::back_inserter(r),
                       [](const T & t)
        {
            return toAron(t);
        });

        return r;
    }

    aron::LaserScanStep toAron(const LaserScanStep& laserScanStep)
    {
        aron::LaserScanStep aronLaserScan;

        aronLaserScan.angle = laserScanStep.angle;
        aronLaserScan.distance = laserScanStep.distance;

        return aronLaserScan;
    }

    auto toAron(const LaserScan& laserScan, aron::LaserScan& aronLaserScan)
    {
        aronLaserScan.scan = toAron(laserScan);
    }

    aron::SensorHeader toAron(const SensorHeader& sensorHeader)
    {
        aron::SensorHeader aronSensorHeader;

        aronSensorHeader.agent = sensorHeader.agent;
        aronSensorHeader.frame = sensorHeader.frame;
        aronSensorHeader.timestamp = sensorHeader.timestamp;

        return aronSensorHeader;
    }

    void toAron(const LaserScanStamped& laserScanStamped,
                aron::LaserScanStamped& aronLaserScanStamped)
    {
        aronLaserScanStamped.header = toAron(laserScanStamped.header);
        toAron(laserScanStamped.data, aronLaserScanStamped.data);
    }

    void toAron(const LaserScan& laserScan,
                const std::int64_t& timestamp,
                const std::string& frame,
                const std::string& agentName,
                aron::LaserScanStamped& aronLaserScanStamped)
    {
        const SensorHeader header
        {
            .agent = agentName, .frame = frame, .timestamp = timestamp};

        const LaserScanStamped laserScanStamped{.header = header, .data = laserScan};

        toAron(laserScanStamped, aronLaserScanStamped);
    }

} // namespace armarx