/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Orientation.h"

namespace armarx::aron::cppcodegenerator::serializer
{
    // constructors
    OrientationSerializer::OrientationSerializer(const typenavigator::OrientationNavigatorPtr& n) :
        Serializer("Eigen::Quaternion<" + n->getTypename() + ">", simox::meta::get_type_name(typeid(data::AronNDArray)), simox::meta::get_type_name(typeid(type::AronOrientation))),
        typenavigator(n)
    {
    }

    std::vector<CppFieldPtr> OrientationSerializer::getPublicVariableDeclarations(const std::string& name) const
    {
        CppFieldPtr field = CppFieldPtr(new CppField(getFullCppTypename(), name));
        return {field};
    }

    std::vector<std::pair<std::string, std::string>> OrientationSerializer::getCtorInitializers(const std::string&) const
    {
        return {};
    }

    CppBlockPtr OrientationSerializer::getCtorBlock(const std::string&) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        return b;
    }

    CppBlockPtr OrientationSerializer::getResetBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + " = " + getFullCppTypename() + "();");
        return b;
    }

    CppBlockPtr OrientationSerializer::getInitializeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine(accessor + " = " + getFullCppTypename() + "();");
        return b;
    }

    CppBlockPtr OrientationSerializer::getWriteInitialTypeBlock(const std::string&) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeOrientation();");
        return b;
    }

    CppBlockPtr OrientationSerializer::getWriteCurrentTypeBlock(const std::string&) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeOrientation();");
        return b;
    }

    CppBlockPtr OrientationSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeNDArray({" + simox::alg::to_string(typenavigator->getDimensions(), ", ") + "}, \"" + typenavigator->getTypename() + "\", reinterpret_cast<const unsigned char*>(" + accessor + ".coeffs().data()));");
        return b;
    }

    CppBlockPtr OrientationSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        std::string escaped_accessor = EscapeAccessor(accessor);
        std::string accessor_dimensions = escaped_accessor + DIMENSION_ACCESSOR;
        std::string accessor_type = escaped_accessor + TYPE_ACCESSOR;

        b->addLine("const auto [" + accessor_dimensions + ", " + accessor_type + "] = r.readStartNDArray();");
        b->addLine(accessor + " = " + getFullCppTypename() + "();");
        b->addLine("r.readEndNDArray(reinterpret_cast<unsigned char*>(" + accessor + ".coeffs().data()));");
        return b;
    }

    CppBlockPtr OrientationSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("if (not (" + accessor + ".isApprox(" + otherInstanceAccessor + ")))");
        b->addLine("\t return false;");
        return b;
    }
}

