/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/Serializer.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/type/enum/IntEnum.h>

namespace armarx::aron::cppcodegenerator::serializer
{

    class IntEnumClassSerializer;
    typedef std::shared_ptr<IntEnumClassSerializer> IntEnumClassSerializerPtr;

    class IntEnumClassSerializer :
        virtual public Serializer
    {
    public:
        using PointerType = IntEnumClassSerializerPtr;

    public:
        // constructors
        IntEnumClassSerializer(const typenavigator::IntEnumNavigatorPtr&);

        // virtual implementations
        std::vector<CppFieldPtr> getPublicVariableDeclarations(const std::string&) const;
        virtual std::vector<std::pair<std::string, std::string>> getCtorInitializers(const std::string&) const override;
        virtual CppBlockPtr getCtorBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteInitialTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteCurrentTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getInitializeBlock(const std::string&) const override;
        virtual CppBlockPtr getResetBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteBlock(const std::string&) const override;
        virtual CppBlockPtr getReadBlock(const std::string&) const override;
        virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;

        CppCtorPtr toCopyCtor(const std::string&) const;
        CppCtorPtr toInnerEnumCtor(const std::string&) const;
        CppEnumPtr toInnerEnumDefinition() const;
        CppMethodPtr toIntMethod() const;
        CppMethodPtr toCopyAssignmentMethod() const;
        CppMethodPtr toEnumAssignmentMethod() const;
        CppMethodPtr toIntAssignmentMethod() const;
        CppMethodPtr toToStringMethod() const;
        CppMethodPtr toFromStringMethod() const;

    private:
        // Members
        static const std::map<std::string, std::pair<std::string, int>> ACCEPTED_TYPES;

        typenavigator::IntEnumNavigatorPtr navigator;
        std::string enumName;
    };
}
