/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Base Class
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/WriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>

namespace armarx::aron::typeIO::writer
{
    class NavigatorWriterToken;
    typedef std::shared_ptr<NavigatorWriterToken> NavigatorWriterTokenPtr;

    class NavigatorWriterToken :
        virtual public WriterTokenInterface<typenavigator::NavigatorPtr>
    {
    public:
        using PointerType = NavigatorWriterTokenPtr;

    public:
        // constructor
        NavigatorWriterToken() = delete;
        NavigatorWriterToken(const type::Descriptor desc, const typenavigator::NavigatorPtr& t)
        {
            descriptor = desc;
            element = t;
        }

        // virtual member functions
        virtual void addElement(const typenavigator::NavigatorPtr& n) override
        {
            switch (descriptor)
            {
                case type::eDict:
                {
                    typenavigator::DictNavigatorPtr casted = typenavigator::DictNavigator::DynamicCastAndCheck(element);
                    casted->setAcceptedType(n);
                    break;
                }
                case type::eList:
                {
                    typenavigator::ListNavigatorPtr casted = typenavigator::ListNavigator::DynamicCastAndCheck(element);
                    casted->setAcceptedType(n);
                    break;
                }
                case type::eObject:
                {
                    typenavigator::ObjectNavigatorPtr casted = typenavigator::ObjectNavigator::DynamicCastAndCheck(element);
                    casted->addMemberType(currentKey, n);
                    break;
                }
                case type::eTuple:
                {
                    typenavigator::TupleNavigatorPtr casted = typenavigator::TupleNavigator::DynamicCastAndCheck(element);
                    casted->addAcceptedType(n);
                    currentIndex++;
                    break;
                }
                case type::ePair:
                {
                    typenavigator::PairNavigatorPtr casted = typenavigator::PairNavigator::DynamicCastAndCheck(element);
                    casted->addAcceptedType(n);
                    currentIndex++;
                    break;
                }
                default:
                    throw error::DescriptorNotValidException("NavigatorWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers. The path was: " + getElement()->pathToString(), descriptor);
            }
        }


        virtual void setName(const std::string& n) override
        {
            switch (descriptor)
            {
                case type::eObject:
                {
                    typenavigator::ObjectNavigatorPtr casted = typenavigator::ObjectNavigator::DynamicCast(element);
                    casted->setObjectName(n);
                    break;
                }
                case type::eIntEnum:
                {
                    typenavigator::IntEnumNavigatorPtr casted = typenavigator::IntEnumNavigator::DynamicCast(element);
                    casted->setEnumName(n);
                    break;
                }
                default:
                    throw error::DescriptorNotValidException("NavigatorWriterToken", "setName", "Cant set the name of a non-object or non-enum token. The path was: " + getElement()->pathToString(), descriptor);

            }
        }

    private:
    };
}
