/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "NavigatorFactory.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>

namespace armarx::aron::typenavigator
{
    // Access method
    NavigatorPtr NavigatorFactory::create(const type::AronTypePtr& aron, const Path& path) const
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    {type::e##upperType, NavigatorFactoryPtr(new upperType##NavigatorFactory())},

        static const std::map<type::Descriptor, NavigatorFactoryPtr> Factories =
        {
            HANDLE_ALL_ARON_TYPES
        };
#undef RUN_ARON_MACRO

        auto typeDescriptor = Resolver::GetDescriptor(aron);
        auto factory_iterator = Factories.find(typeDescriptor);
        if (factory_iterator == Factories.end())
        {
            throw error::AronException("NavigatorFactory", "create", "Cannot find the desired factory for input: " + type::DESCRIPTOR_TO_STRING(typeDescriptor) + ". Cannot create navigator", path);
        }
        return factory_iterator->second->createSpecific(aron, path);
    }

    NavigatorPtr NavigatorFactory::createSpecific(const type::AronTypePtr&, const Path& p) const
    {
        throw error::AronException("NavigatorFactory", "createSpecific", "Called disallowed method of an NavigatorFactory. Use child class instead!", p);
    }

    // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    NavigatorPtr upperType##NavigatorFactory::createSpecific(const type::AronTypePtr& aron, const Path& path) const \
    { \
        type::Aron##upperType##Ptr aronCasted = type::Aron##upperType##Ptr::dynamicCast(aron); \
        return typenavigator::NavigatorPtr(new upperType##Navigator(aronCasted, path)); \
    }

    HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO

}
