/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Orientation.h"

namespace armarx::aron::typenavigator
{
    // constructors
    OrientationNavigator::OrientationNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eOrientation, path),
        Navigator(type::Descriptor::eOrientation, path),
        type(new type::AronOrientation())
    {
    }

    OrientationNavigator::OrientationNavigator(const type::AronOrientationPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eOrientation, path),
        Navigator(type::Descriptor::eOrientation, path),
        type(o)
    {
        CheckAronPtrForNull("AronOrientationTypeNavigator", "AronOrientationTypeNavigator", getPath(), o);
    }

    type::AronOrientationPtr OrientationNavigator::toAronOrientationPtr() const
    {
        return type;
    }

    std::vector<int> OrientationNavigator::getDimensions() const
    {
        return ACCEPTED_DIMENSION;
    }

    std::string OrientationNavigator::getTypename() const
    {
        return ACCEPTED_TYPE;
    }

    // static methods
    OrientationNavigatorPtr OrientationNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<OrientationNavigator>(n);
    }

    OrientationNavigatorPtr OrientationNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronOrientationTypeNavigator", "DynamicCast[Before]", n);
        OrientationNavigatorPtr casted = OrientationNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronOrientationTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> OrientationNavigator::getChildren() const
    {
        return {};
    }

    size_t OrientationNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr OrientationNavigator::getResult() const
    {
        return toAronOrientationPtr();
    }

    std::string OrientationNavigator::getName() const
    {
        return "AronOrientationType";
    }
}

