/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class PositionNavigator;
    typedef std::shared_ptr<PositionNavigator> PositionNavigatorPtr;

    class PositionNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = PositionNavigatorPtr;

    public:
        // constructors
        PositionNavigator(const Path& path = Path());
        PositionNavigator(const type::AronPositionPtr&, const Path& path = Path());

        std::string getTypename() const;
        std::vector<int> getDimensions() const;

        type::AronPositionPtr toAronPositionPtr() const;

        // static methods
        static PositionNavigatorPtr DynamicCast(const NavigatorPtr&);
        static PositionNavigatorPtr DynamicCastAndCheck(const NavigatorPtr&);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        type::AronTypePtr getResult() const override;
        std::string getName() const override;

    public:
        const std::string ACCEPTED_TYPE = "float";
        const std::vector<int> ACCEPTED_DIMENSION = {3, 1};

    private:
        // members
        type::AronPositionPtr type;
    };
}
