/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */


// STD/STL
#include <string>
#include <map>

// Header
#include "Primitive.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/Concepts.h>

namespace armarx::aron::typenavigator
{
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    /* constructors */ \
    upperType##Navigator::upperType##Navigator(const Path& path) : \
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::e##upperType, path), \
        Navigator(type::Descriptor::e##upperType, path), \
        type(new type::Aron##upperType()) \
    { \
    } \
    \
    upperType##Navigator::upperType##Navigator(const type::Aron##upperType##Ptr&o, const Path& path) : \
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::e##upperType, path), \
        Navigator(type::Descriptor::e##upperType, path), \
        type(o) \
    { \
        CheckAronPtrForNull("typenavigator::" + std::string(#upperType) + "Navigator", std::string(#upperType) + "Navigator", getPath(), o); \
    } \
    \
    type::Aron##upperType##Ptr upperType##Navigator::toAron##upperType##Ptr() const \
    { \
        return type; \
    } \
    \
    /* static methods */ \
    upperType##NavigatorPtr upperType##Navigator::DynamicCastAndCheck(const NavigatorPtr& n) \
    {\
        CheckTypeNavigatorPtrForNull("typenavigator::" + std::string(#upperType) + "Navigator", "DynamicCast[Before]", n); \
        upperType##NavigatorPtr casted = upperType##Navigator::DynamicCast(n); \
        CheckTypeNavigatorPtrForNull("typenavigator::" + std::string(#upperType) + "Navigator", "DynamicCast[After]", n->getPath(), casted); \
        return casted; \
    }\
    \
    upperType##NavigatorPtr upperType##Navigator::DynamicCast(const NavigatorPtr& n) \
    {\
        return std::dynamic_pointer_cast<upperType##Navigator>(n); \
    }\
    \
    /* virtual implementations */ \
    std::vector<NavigatorPtr> upperType##Navigator::getChildren() const \
    { \
        return {}; \
    } \
    \
    size_t upperType##Navigator::childrenSize() const \
    { \
        return 0; \
    } \
    \
    type::AronTypePtr upperType##Navigator::getResult() const \
    {\
        return toAron##upperType##Ptr(); \
    }\
    \
    std::string upperType##Navigator::getName() const \
    {\
        return "type::Aron" + std::string(#upperType); \
    }

    HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
}
