#pragma once

#include <vector>

#include <RobotAPI/libraries/aron/core/io/dataIO/writer/navigator/NavigatorWriter.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/reader/navigator/NavigatorReader.h>

#include <RobotAPI/libraries/aron/core/io/dataIO/writer/navigator/NavigatorWriter.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/reader/navigator/NavigatorReader.h>

#include <RobotAPI/libraries/aron/core/io/typeIO/writer/Writer.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/navigator/NavigatorWriter.h>

namespace armarx
{

    class AronTrivialDataWithoutCodeGeneration
    {
    public:
        float f = 1;
        int   i = 42;

        bool operator==(const AronTrivialDataWithoutCodeGeneration& x) const
        {
            return x.f == f && x.i == i;
        }

        void write(armarx::aron::io::AronDataWriter& w) const
        {
            w.writeStartDict();
            w.writeKey("f");
            w.writePrimitive(f);
            w.writeKey("i");
            w.writePrimitive(i);
            w.writeEndDict();
        }

        void writeType(armarx::aron::io::Writer& w) const
        {
            w.writeStartObjectType();
            w.writeObjectName("AronTrivialDataWithoutCodeGeneration");
            w.writeKey("f");
            w.writeFloatType();
            w.writeKey("i");
            w.writeIntType();
            w.writeEndObjectType();
        }

        void read(armarx::aron::io::AronDataReader& w)
        {
            f = 0;
            i = 0;

            w.readStartDict();
            w.readKey("f");
            w.readPrimitive(f);
            w.readKey("i");
            w.readPrimitive(i);
            w.readEndDict();
        }

        armarx::aron::datanavigator::DictNavigatorPtr toAron() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->write(writer);
            return armarx::aron::datanavigator::DictNavigator::DynamicCast(writer.getResult());
        }

        void fromAron(const armarx::aron::datanavigator::DictNavigatorPtr& input)
        {
            armarx::aron::io::NavigatorReader reader(input);
            //this->read(reader);
        }

        armarx::aron::typenavigator::ObjectNavigatorPtr toInitialAronType() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->writeType(writer);
            return armarx::aron::typenavigator::ObjectNavigator::DynamicCast(writer.getResult());
        }
    };

    class AronSimpleDataWithoutCodeGeneration
    {
    public:
        std::string      str   = "stringy";
        std::vector<int> vec_i = {37, 38, 39};

        bool operator==(const AronSimpleDataWithoutCodeGeneration& i) const
        {
            return i.str == str && i.vec_i == vec_i;
        }

        void write(armarx::aron::io::AronDataWriter& w) const
        {
            w.writeStartDict();

            w.writeKey("str");
            w.writePrimitive(str);

            w.writeKey("vec_i");
            w.writeStartList();
            for (const auto& e : vec_i)
            {
                w.writePrimitive(e);
            }
            w.writeEndList();

            w.writeEndDict();
        }

        void writeType(armarx::aron::io::Writer& w) const
        {
            w.writeStartObjectType();
            w.writeObjectName("AronSimpleDataWithoutCodeGeneration");
            w.writeKey("str");
            w.writeStringType();
            w.writeKey("vec_i");
            w.writeStartListType();
            w.writeIntType();
            w.writeEndListType();
            w.writeEndObjectType();
        }

        void read(armarx::aron::io::AronDataReader& w)
        {
            str = "";
            vec_i.clear();

            w.readStartDict();

            w.readKey("str");
            w.readPrimitive(str);

            w.readKey("vec_i");
            w.readStartList();
            while (!w.readEndList())
            {
                int e;
                w.readPrimitive(e);
                vec_i.push_back(e);
            }
            w.readEndDict();
        }


        armarx::aron::datanavigator::DictNavigatorPtr toAron() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->write(writer);
            return armarx::aron::datanavigator::DictNavigator::DynamicCast(writer.getResult());
        }

        void fromAron(const armarx::aron::datanavigator::DictNavigatorPtr& input)
        {
            //armarx::aron::io::NavigatorReader reader(input);
            //this->read(reader);
        }

        armarx::aron::typenavigator::ObjectNavigatorPtr toInitialAronType() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->writeType(writer);
            return armarx::aron::typenavigator::ObjectNavigator::DynamicCast(writer.getResult());
        }
    };

    class AronNestedDataWithoutCodeGeneration
    {
    public:
        AronTrivialDataWithoutCodeGeneration tri;
        std::vector<AronTrivialDataWithoutCodeGeneration> vec_tri{{}, {}, {}};
        AronSimpleDataWithoutCodeGeneration simp;
        std::vector<AronSimpleDataWithoutCodeGeneration> vec_simp{{}, {}};

        bool operator==(const AronNestedDataWithoutCodeGeneration& i) const
        {
            return i.tri == tri &&
                   i.vec_tri == vec_tri &&
                   simp == i.simp &&
                   i.vec_simp == vec_simp;
        }

        void write(armarx::aron::io::AronDataWriter& w) const
        {
            w.writeStartDict();

            w.writeKey("tri");
            tri.write(w);

            w.writeKey("vec_tri");
            w.writeStartList();
            for (auto& e : vec_tri)
            {
                e.write(w);
            }
            w.writeEndList();

            w.writeKey("simp");
            simp.write(w);
            w.writeKey("vec_simp");
            w.writeStartList();
            for (auto& e : vec_simp)
            {
                e.write(w);
            }
            w.writeEndList();
            w.writeEndDict();
        }

        void writeType(armarx::aron::io::Writer& w) const
        {
            w.writeStartObjectType();
            w.writeObjectName("AronNestedDataWithoutCodeGeneration");
            w.writeKey("tri");
            tri.writeType(w);

            w.writeKey("vec_tri");
            w.writeStartListType();
            tri.writeType(w); // hack here
            w.writeEndListType();

            w.writeKey("simp");
            simp.writeType(w);

            w.writeKey("vec_simp");
            w.writeStartListType();
            simp.writeType(w); // hack here
            w.writeEndListType();

            w.writeEndObjectType();
        }

        void read(armarx::aron::io::AronDataReader& w)
        {
            // dont forget to reset!
            tri = AronTrivialDataWithoutCodeGeneration();
            vec_tri.clear();
            simp = AronSimpleDataWithoutCodeGeneration();
            vec_simp.clear();

            w.readStartDict();

            w.readKey("tri");
            tri.read(w);

            w.readKey("vec_tri");
            w.readStartList();
            while (!w.readEndList())
            {
                AronTrivialDataWithoutCodeGeneration e;
                e.read(w);
                vec_tri.push_back(e);
            }

            w.readKey("simp");
            simp.read(w);

            w.readKey("vec_simp");
            w.readStartList();
            while (!w.readEndList())
            {
                AronSimpleDataWithoutCodeGeneration e;
                e.read(w);
                vec_simp.push_back(e);
            }
            w.readEndDict();
        }

        armarx::aron::datanavigator::DictNavigatorPtr toAron() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->write(writer);
            return armarx::aron::datanavigator::DictNavigator::DynamicCast(writer.getResult());
        }

        void fromAron(const armarx::aron::datanavigator::DictNavigatorPtr& input)
        {
            armarx::aron::io::NavigatorReader reader(input);
            //this->read(reader);
        }

        armarx::aron::typenavigator::ObjectNavigatorPtr toInitialAronType() const
        {
            armarx::aron::io::NavigatorWriter writer;
            this->writeType(writer);
            return armarx::aron::typenavigator::ObjectNavigator::DynamicCast(writer.getResult());
        }

    };

    /*template<class T>
    void test(const auto& name)
    {
        std::cout << "############################\n" << name << '\n';
        T orig;
        T round;
        std::string orig_j;
        std::string round_j;

        {
            armarx::aron::io::AronDataJSONWriter jwrt_o;
            write(orig, jwrt_o);
            orig_j = jwrt_o.getResult();
            std::cout << "orig\n" << orig_j << '\n';
        }

        {
            armarx::aron::io::NavigatorWriter wrt;
            write(orig, wrt);
            armarx::aron::io::NavigatorReader rdr(wrt.getResult());
            read(round, rdr);
        }

        {
            armarx::aron::io::AronDataJSONWriter jwrt_r;
            write(round, jwrt_r);
            round_j = jwrt_r.getResult();
            std::cout << "round\n" << round_j << '\n';
        }

        std::cout << "equal? " << (orig_j == round_j) << '\n';
    }*/

    //#include <iostream>
    //int main()
    //{
    //    test<trivial>("trivial");
    //    test<simple >("simple");
    //    test<nested >("nested");
    //
    //    return 0;
    //}

}
